import Clutter from 'gi://Clutter';
import Meta from 'gi://Meta';
import * as Main from 'resource:///org/gnome/shell/ui/main.js';
import { Extension } from 'resource:///org/gnome/shell/extensions/extension.js';
const BACKGROUND_FACTOR = 'desaturation-factor-background';
const FOREGROUND_FACTOR = 'desaturation-factor-foreground';
const DESATURATE_OVERVIEW = 'desaturation-in-overview';
const EFFECT_NAME = 'window-desaturation@mkhl.codeberg.page';
export default class WindowDesaturation extends Extension {
    connections;
    focused_window;
    settings;
    enable() {
        this.settings = this.getSettings();
        this.focused_window = global.display.focus_window;
        for (const window of this.windows()) {
            this.handle(window);
        }
        this.settings.connect('changed', () => this.refresh());
        this.connections = new Map();
        this.connections.set(global.display, [
            global.display.connect('window-created', (_, window) => this.handle(window)),
            global.display.connect('focus-window', (_, window) => this.update_focus(window)),
        ]);
        this.connections.set(Main.overview, [
            Main.overview.connect('showing', () => this.refresh()),
            Main.overview.connect('hidden', () => this.refresh()),
        ]);
    }
    disable() {
        this.settings = undefined;
        for (const [obj, ids] of this.connections.entries()) {
            for (const id of ids) {
                obj.disconnect(id);
            }
        }
        this.connections.clear();
        this.connections = undefined;
        for (const window of this.windows()) {
            this.clear(window);
        }
    }
    refresh() {
        for (const window of this.windows()) {
            this.update(window);
        }
    }
    handle(window) {
        if (window.window_type !== Meta.WindowType.NORMAL)
            return;
        const actor = this.actor(window);
        if (!actor)
            return;
        const factor = this.factor(window);
        const effect = new Clutter.DesaturateEffect({ factor });
        actor.add_effect_with_name(EFFECT_NAME, effect);
    }
    clear(window) {
        const actor = this.actor(window);
        if (!actor)
            return;
        actor.remove_effect_by_name(EFFECT_NAME);
    }
    update_focus(window) {
        if (this.focused_window === window)
            return;
        this.focus(window);
        this.defocus(this.focused_window);
        this.focused_window = window;
    }
    focus(window) {
        const effect = this.effect(window);
        if (!effect)
            return;
        effect.factor = this.foreground_factor();
    }
    defocus(window) {
        const effect = this.effect(window);
        if (!effect)
            return;
        effect.factor = this.background_factor();
    }
    update(window) {
        const effect = this.effect(window);
        if (!effect)
            return;
        effect.factor = this.factor(window);
    }
    actor(window) {
        if (!window)
            return;
        return window.get_compositor_private();
    }
    effect(window) {
        if (!window)
            return;
        const actor = this.actor(window);
        if (!actor)
            return;
        const effect = actor.get_effect(EFFECT_NAME);
        if (!(effect instanceof Clutter.DesaturateEffect))
            return;
        return effect;
    }
    factor(window) {
        return Main.overview.visible && !this.desaturate_overview()
            ? 0
            : window === global.display.focus_window
                ? this.foreground_factor()
                : this.background_factor();
    }
    windows() {
        return global.display.list_all_windows();
    }
    background_factor() {
        return this.settings.get_double(BACKGROUND_FACTOR);
    }
    foreground_factor() {
        return this.settings.get_double(FOREGROUND_FACTOR);
    }
    desaturate_overview() {
        return this.settings.get_boolean(DESATURATE_OVERVIEW);
    }
}
